<?php

namespace App\Models;

use Carbon\Carbon;
use Illuminate\Database\Eloquent\Factories\HasFactory;
use Illuminate\Database\Eloquent\Model;

class LoadBillingInvoice extends Model
{
    use HasFactory;

    protected $table = 'load_billing_invoice';

    /**
     * The attributes that are mass assignable.
     *
     * @var array
     */
    protected $fillable = [
        'load_id',
        'load_detail_id',
        'invoice_number',
        'payments_terms_date',
        'payments_terms_days',
        'invoice_template',
        'invoice_date',
        'due_date',
        'paid_date',
        'billing_method',
        'invoice_email',
        'invoice_email_cc',
        'invoice_email_bcc',

        'customer',
        'bill_to_name',
        'billing_address',
        'billing_address_2',
        'billing_city',
        'billing_state',
        'billing_zip',
        'po_number',
        'bol_number',
        'pickup_number',
        'delivery_number',
        'notes',

        'add_qr_code',
        'send_payment_reminder',
        'payment_reminder_days',

        'salesman',
        'carrier',
        'dispatcher',
        'truck',
        'trailer',
        'driver',

        'invoice_amount',
        'invoice_created_by',
        'invoice_created_on',
    ];

    public function setInvoiceNumberAttribute($value): ?string {
        $maxInvoiceNumber = 0;
        
        $companyIdentifier = getCompIdentifier();

        $invoiceNumbers = LoadBillingInvoice::where('invoice_number', 'LIKE', $companyIdentifier.'%')->pluck('invoice_number')->toArray();

        if(count($invoiceNumbers) > 0){
            // Extract numbers from filtered strings
            $invoiceNumbersArr = array_map(function($str) {
                preg_match('/(\d+)/', $str, $matches);
                return intval($matches[0]);
            }, $invoiceNumbers);

            // Find maximum number
            $maxInvoiceNumber = max($invoiceNumbersArr);
        }

        $nextInvoiceNumber = ($maxInvoiceNumber > 0) ? ($maxInvoiceNumber + 1) : 1;

        return $this->attributes['invoice_number'] = $companyIdentifier.$nextInvoiceNumber;
    }

    public function setInvoiceDateAttribute($value): ?string {
        return $this->attributes['invoice_date'] = !is_null($value) ? Carbon::createFromFormat(config('settings.date_format','m/d/Y'),$value)->format('Y-m-d') : NULL;
    }

    public function getInvoiceDateAttribute($value): ?string{
        return !is_null($value) ? Carbon::parse($value)->timezone(getTimeZone())->format('m/d/Y') : NULL;
    }

    public function setDueDateAttribute($value): ?string {
        return $this->attributes['due_date'] = !is_null($value) ? Carbon::createFromFormat(config('settings.date_format','m/d/Y'),$value)->format('Y-m-d') : NULL;
    }

    public function getDueDateAttribute($value): ?string{
        return !is_null($value) ? Carbon::parse($value)->timezone(getTimeZone())->format('m/d/Y') : NULL;
    }

    public function setPaidDateAttribute($value): ?string {
        return $this->attributes['paid_date'] = !is_null($value) ? Carbon::createFromFormat('m/d/Y H:i:s', $value)->format('Y-m-d H:i:s') : NULL;
    }

    public function getPaidDateAttribute($value): ?string{
        return !is_null($value) ? Carbon::parse($value)->timezone(getTimeZone())->format('m/d/Y H:i:s') : NULL;
    }

    /*
     * Relationships
     */
    public function loads(){
        // 'load_id' is the custom foreign key column in the load_document_requests table
        // 'id' is the default primary key column in the loads table
        return $this->belongsTo(Load::class,'load_id','id');
    }

    public function load_detail(){
        // 'load_detail_id' is the custom foreign key column in the load_document_requests table
        // 'id' is the default primary key column in the load_details table
        return $this->hasOne(LoadDetail::class,'id','load_detail_id');
    }

    public function broker(){
        return $this->hasOne(Broker::class,'id','customer')->withTrashed();
    }

    public function billingFullAddress() {
        $billingFullAddress = '';
        if($this->attributes['billing_address'] != ''){
            $billingFullAddress .= $this->attributes['billing_address'].',<br>';
        }
        if($this->attributes['billing_address_2'] != ''){
            $billingFullAddress .= $this->attributes['billing_address_2'].', ';
        }
        if($this->attributes['billing_city'] != ''){
            $billingFullAddress .= $this->attributes['billing_city'].', ';
        }
        if($this->attributes['billing_state'] != ''){
            $billingFullAddress .= $this->attributes['billing_state'].', ';
        }
        if($this->attributes['billing_zip'] != ''){
            $billingFullAddress .= $this->attributes['billing_zip'];
        }
        return $billingFullAddress;
    }
}
