<?php

namespace App\Models;

use Illuminate\Database\Eloquent\Factories\HasFactory;
use Illuminate\Database\Eloquent\Model;

use Illuminate\Database\Eloquent\SoftDeletes;

use Carbon\Carbon;

class LoadStop extends Model
{
    use HasFactory, SoftDeletes;

    protected $table        = 'load_stops';
    protected $fillable     = [
        'stop_no',
        'company_id',
        'load_id',

        'date_time_range',
        'date_time',
        'date_time_end',

        'customer_id',
        'customer_type',

        'location',
        'address_one',
        'address_two',
        'state',
        'city',
        'zip',
        'latitude',
        'longitude',
        'timezone',
        'timezone_offset',
        'timezone_abbr',
        'address_verified',

        'stop_note',

        'commodity',
        'stop_value',
        'stop_weight',
        'reefer_mode',
        'reefer_temp_type',
        'reefer_temp',
        'pickup_number',
        'dimension',

        'drop_trailer_schedule',

        'drop_trailer_id',
        'verified_drop_trailer_id',
        'drop_trailer_match',
        'drop_trailer_reminder_on',
        'drop_trailer_reminder_date_time',
        'drop_trailer_reminder_status',
        'drop_trailer_files',
        'drop_trailer_confirm',
        'drop_trailer_confirm_at',

        'pickup_trailer_id',
        'verified_pickup_trailer_id',
        'pickup_trailer_match',
        'pickup_trailer_files',
        'pickup_trailer_confirm',
        'pickup_trailer_confirm_at',

        'stop_shipper_files',
        'stop_shipper_files_upload',
        'stop_consignee_files',
        'stop_consignee_files_upload',

        'reason',
        'status',
        'status_at',

        'is_arrived',
        'arrived_at',
        'is_leaves',
        'leaves_at',

        'created_at',
        'updated_at',
        'deleted_at'
    ];

    protected $dates        = ['deleted_at'];
    // public $timestamps   = false;

    // this is a recommended way to declare event handlers
    public static function boot() {
        parent::boot();

        static::created(function($model){
            if($model->isDirty('stop_shipper_files_upload') || $model->isDirty('stop_consignee_files_upload')) {
                updateLoadBillingActivity($model->loads->load_detail->id);
            }
        });

        static::updated(function($model){
            if($model->isDirty('stop_shipper_files_upload') || $model->isDirty('stop_consignee_files_upload')) {
                updateLoadBillingActivity($model->loads->load_detail->id);
            }
        });

        static::deleting(function($stop) {
            // before delete() method call this
            $stop->appointment()->delete();
            $stop->detention()->delete();
            $stop->accessorials()->delete();
            $stop->load_document_request_items()->delete();
            // do the rest of the cleanup...
        });
    }

    /*
     * Scopes
     */
    public function scopeCompany($query){
        return $query->where('company_id', auth()->guard('company-admin')->user()->company_id);
    }

    /*
     * Mutators
     */
    public function setDateTimeAttribute($value): ?string {
        return $this->attributes['date_time']               = !is_null($value) ? Carbon::createFromFormat(getTimeLayout(),$value)->format('Y-m-d H:i:s') : NULL;
    }

    public function setDateTimeEndAttribute($value): ?string {
        return $this->attributes['date_time_end']           = !is_null($value) ? Carbon::createFromFormat(getTimeLayout(),$value)->format('Y-m-d H:i:s') : NULL;
    }

    public function setStopValueAttribute($value){
        $valueWithoutCommas                                 = null;
        if(!is_null($value)){
            // Remove commas from the value
            $valueWithoutCommas                             = str_replace(',', '', $value);
            $valueWithoutCommas                             = (float) $valueWithoutCommas;
        }
        $this->attributes['stop_value']                     = $valueWithoutCommas;
    }

    public function setStopWeightAttribute($value){
        $valueWithoutCommas                                 = null;
        if(!is_null($value)){
            // Remove commas from the value
            $valueWithoutCommas                             = str_replace(',', '', $value);
            $valueWithoutCommas                             = (float) $valueWithoutCommas;
        }
        $this->attributes['stop_weight']                    = $valueWithoutCommas;
    }

    /*
     * Accessors
     */
    public function getDateTimeAttribute($value): ?string {
        return !is_null($value) ? Carbon::parse($value)->format(getTimeLayout()) : null;
    }

    public function getDateTimeEndAttribute($value): ?string {
        return !is_null($value) ? Carbon::parse($value)->format(getTimeLayout()) : null;
    }

    /*
     * Relationships
     */
    public function loads(){
        return $this->belongsTo(Load::class,'load_id','id');
    }

    public function customer(){
        return $this->hasOne(Customer::class, 'id', 'customer_id')->withTrashed();
    }

    public function load_commodity(){
        return $this->hasOne(CompanyCommodity::class,'id','commodity')->withTrashed();
    }

    public function load_document_request_items(){
        return $this->hasMany(LoadDocumentRequestItem::class,'load_stop_id','id')->where('is_stop', 1);
    }

    public function appointment(){
        return $this->hasOne(LoadAppointment::class,'load_stop_id','id')->where('is_stop', 1);
    }

    public function detention(){
        return $this->hasOne(LoadDetention::class,'load_stop_id','id')->where('is_stop', 1);
    }

    public function accessorials(){
        return $this->hasMany(LoadAccessorial::class,'load_stop_id','id')->where('is_stop', 1)->where('type', 3);
    }

    public function review(){
        return $this->hasOne(LoadStopReview::class,'load_stop_id','id')->where('is_stop', 1);
    }
}
